#!/bin/bash

#ip route del default via 192.168.0.252
#ip route add default via 192.168.0.252


#Black        0;30     Dark Gray     1;30
#Red          0;31     Light Red     1;31
#Green        0;32     Light Green   1;32
#Brown/Orange 0;33     Yellow        1;33
#Blue         0;34     Light Blue    1;34
#Purple       0;35     Light Purple  1;35
#Cyan         0;36     Light Cyan    1;36
#Light Gray   0;37     White         1;37

BLACK='\033[0;30m'
RED='\033[0;31m'
GREEN='\033[0;32m'
ORAN='\033[0;33m'
BLUE='\033[0;34m'
PURP='\033[0;35m'
CYAN='\033[0;36m'
LGRAY='\033[0;37m'
DGRAY='\033[1;30m'
LRED='\033[1;31m'
LGR='\033[1;32m'
YEL='\033[1;33m'
LBL='\033[1;34m'
LPU='\033[1;35m'
LCY='\033[1;36m'
WHI='\033[1;37m'
NC='\033[0m'

LASTSEEN=$(date +%s)
echo $LASTSEEN > /root/routing/.netBalancer/lastSeen

A_RTR='192.168.0.251'
B_RTR='192.168.0.252'

PERF_API_URL='/api/netPerfs.json'
DATA_API_URL='/api/netData.json'

MANIPULATION_COOLDOWN=60

SILENCE=0

#JQ_STATUS="| jq -r '.server_1 | .status'"
#JQ_MAX_DT="| jq '.data_max_gb'"
#JQ_ACT_DT="| jq '.gb'"

## Examples
# curl -s http://192.168.0.251/api/netData.json | jq '.data_max_gb'
# curl -s 'http://localhost/api/netPerfs.json' | jq -r '.server_1 | .status'

if [[ $1 == 'help' ]]; then
	echo -e "help : \n\tdistributed\t: will distribute between both 4G modems so consumption will go accordingnly.\n\tfallback <a|b>\t: will make sure that network will stay up and running at all time by netbalancing to slave node,\n\t\t\t  if network is OK on both nodes, the selected node will be the master and selected as default. By default this is Node A."
	exit 1
	#	- distributed		: will distribute between both 4G modems so consumption will go accordingnly
#	- fallback <a/b>	: will make sure that network will stay up and running at all time by netbalancing to slave node, 
#						  if network is OK on both nodes, the selected node will be the master and selected as default. By default this is Node A.
fi

#default options, 1 can be Dist/Fallback, 2 can be A/B or Silence, 3 can be silence.
if [[ $1 == '' ]]; then
	OPT='distributed'
else
	# can be Distributed or Fallback, if 2 == a or b then Arg =2
	if [[ $2 == '--silence' ]]; then
		ARG=''
		SILENCE=1
	else
		ARG=$2
	fi

	# if 3 == silence then we have 2 opts: opt et arg.
	if [[ $3 == '--silence' ]]; then
		SILENCE=1
		ARG=$2
	fi

	OPT=$1
fi

echo $OPT > /root/routing/.netBalancer/mode

A_RTR_CURL_URL=$(echo "http://"$A_RTR$PERF_API_URL)
B_RTR_CURL_URL=$(echo "http://"$B_RTR$PERF_API_URL)

A_RTR_CURL_DATA_URL=$(echo "http://"$A_RTR$DATA_API_URL)
B_RTR_CURL_DATA_URL=$(echo "http://"$B_RTR$DATA_API_URL)

# TODO : Edit URLs - need to point towards new routerNetData.json
# TODO : Also we need to update facturation date to 13rd/month for orange.


#/usr/bin/curl --show-error --connec-timeout 1 -s $P_RTR_CURL_URL
A_RTR_CURL=$(/usr/bin/curl --connect-timeout 1 -s $A_RTR_CURL_URL)
B_RTR_CURL=$(/usr/bin/curl --connect-timeout 1 -s $B_RTR_CURL_URL)

if [[ $1 == 'debug' ]]; then echo -e "A: "$A_RTR_CURL_URL; echo -e "\t--> "$A_RTR_CURL; fi
if [[ $1 == 'debug' ]]; then echo -e "B: "$B_RTR_CURL_URL; echo -e "\t--> "$B_RTR_CURL; fi

A_RTR_CURL_PING_STATUS=$(echo $A_RTR_CURL | jq -r '.server_1 | .status')
B_RTR_CURL_PING_STATUS=$(echo $B_RTR_CURL | jq -r '.server_1 | .status')

if [[ $1 == 'debug' ]]; then echo -e "A: "$A_RTR" - status reading: "$A_RTR_CURL_PING_STATUS; fi
if [[ $1 == 'debug' ]]; then echo -e "B: "$B_RTR" - status reading: "$B_RTR_CURL_PING_STATUS; fi

function logger () {
	__DATELOG=$(date +'%Y-%m-%d %H:%M:%S')
	echo -e $__DATELOG' '$1 >> /root/logs/netBalancer.log

	if [[ $SILENCE -eq 0 ]]; then
		echo -e $1
	fi
}


# Pinging hosts
PING_A=$(ping -qc1 $A_RTR -w 1 -I eth0 2>&1 | awk -F'/' 'END{ print (/^rtt/? "OK "$5" ms":"FAIL") }')
PING_B=$(ping -qc1 $B_RTR -w 1 -I eth0 2>&1 | awk -F'/' 'END{ print (/^rtt/? "OK "$5" ms":"FAIL") }')

PING_A_RES=$(echo $PING_A | cut -d ' ' -f 1)
PING_B_RES=$(echo $PING_B | cut -d ' ' -f 1)
PING_A_MS=$(echo $PING_A | cut -d ' ' -f 2)
PING_B_MS=$(echo $PING_B | cut -d ' ' -f 2)


## Net Connectivity
A_UNSTABLE_SCORE=0
B_UNSTABLE_SCORE=0

# Testing A net connectivity
if [[ $PING_A_RES == 'OK' ]]; then
	# Ping OK, checking details
	#logger 'Node A: Host Ping '$PING_A
	if [[ $A_RTR_CURL != '' ]]; then
		if [[ $SILENCE -eq 0 ]]; then echo -ne "Node A:"; fi
		if [[ $A_RTR_CURL_PING_STATUS == "OK" ]]; then
			if [[ $SILENCE -eq 0 ]]; then echo -e "\tAPI: OK!"; fi
			# OK - Reset to 0
			A_UNSTABLE_SCORE=0
			logger 'Node A: '$PING_A_RES' '$PING_A_MS'ms \t| GG_PING: OK - Unstability factor: '$A_UNSTABLE_SCORE'/5'

		else
			for i in `seq 1 5`
			do
				A_RTR_CURL_PING_STATUS=$(echo $A_RTR_CURL | jq -r '.server_1 | .status')

            			if [[ $A_RTR_CURL_PING_STATUS == "FAIL" ]] || [[ $A_RTR_CURL == '' ]]; then
                    			A_UNSTABLE_SCORE=$((A_UNSTABLE_SCORE+1))
            			elif [[ $A_RTR_CURL_PING_STATUS == "OK" ]]; then
                    			A_UNSTABLE_SCORE=0
                    			break
            			fi

            			if [[ $SILENCE -eq 0 ]]; then echo -e "\tTry $i/5: "$A_RTR_CURL_PING_STATUS" - Score: "$A_UNSTABLE_SCORE; fi
            			sleep 1
	        	done
	        	logger 'Node A: GG_PING Status is bad - Unstability factor: '$A_UNSTABLE_SCORE'/5'
	        	#At this point here, we should be either at 5 or somewhere within the 0-5 range (if break cause OK).
		fi
	else
        	A_UNSTABLE_SCORE=5
		logger 'Node A: API Error: API from '$B_RTR' is not answering... - Unstability factor: '$A_UNSTABLE_SCORE'/5'
		if [[ $SILENCE -eq 0 ]]; then echo "ERRM: API from $A_RTR is not answering..."; fi
	fi
else
	#ping KO, passing
	logger 'Node A: Host Ping FAIL'
	A_UNSTABLE_SCORE=10
fi

# Testing B net connectivity...
if [[ $PING_B_RES == 'OK' ]]; then
	# Ping OK, checking details
	#logger 'Node B: Host Ping '$PING_B

	if [[ $B_RTR_CURL != '' ]]; then
		if [[ $SILENCE -eq 0 ]]; then echo -ne "Node B:"; fi
		if [[ $B_RTR_CURL_PING_STATUS == "OK" ]]; then
				if [[ $SILENCE -eq 0 ]]; then echo -e "\tAPI: OK!"; fi
	        	# OK - Reset to 0
	        	B_UNSTABLE_SCORE=0
				logger 'Node B: '$PING_B_RES' '$PING_B_MS'ms \t| GG_PING: OK - Unstability factor: '$B_UNSTABLE_SCORE'/5'

		else
			for i in `seq 1 5`
			do
				B_RTR_CURL_PING_STATUS=$(echo $B_RTR_CURL | jq -r '.server_1 | .status')

				if [[ $B_RTR_CURL_PING_STATUS == "FAIL" ]] || [[ $B_RTR_CURL == '' ]]; then
					B_UNSTABLE_SCORE=$((B_UNSTABLE_SCORE+1))
				elif [[ $B_RTR_CURL_PING_STATUS == "OK" ]]; then
					B_UNSTABLE_SCORE=0
					break
				fi

	        	if [[ $SILENCE -eq 0 ]]; then echo -e "\tTry $i/5: "$B_RTR_CURL_PING_STATUS" - Score: "$B_UNSTABLE_SCORE; fi
				sleep 1
			done
			logger 'Node B: GG_PING Status is bad - Unstability factor: '$B_UNSTABLE_SCORE'/5'
			#At this point here, we should be either at 5 or somewhere within the 0-5 range (if break cause OK).
		fi
	else
                B_UNSTABLE_SCORE=5
		logger 'Node B: API Error: API from '$B_RTR' is not answering... - Unstability factor: '$B_UNSTABLE_SCORE'/5'
	        if [[ $SILENCE -eq 0 ]]; then echo "ERRM: API from $B_RTR is not answering..."; fi
	fi
else
	#ping KO, passing
	logger 'Node B: Host Ping FAIL'
	B_UNSTABLE_SCORE=10
fi

## Find less used Provider
function findLessUsedProvider {
	A_RTR_CURL_DATA=$(/usr/bin/curl --connect-timeout 1 -s $A_RTR_CURL_DATA_URL)
	B_RTR_CURL_DATA=$(/usr/bin/curl --connect-timeout 1 -s $B_RTR_CURL_DATA_URL)

	if [[ $1 == 'debug' ]]; then echo -e "A: "$A_RTR_CURL_DATA_URL; fi; if [[ $1 == 'debug' ]]; then echo -e "\t--> "$A_RTR_CURL_DATA; fi
	if [[ $1 == 'debug' ]]; then echo -e "B: "$B_RTR_CURL_DATA_URL; fi; if [[ $1 == 'debug' ]]; then echo -e "\t--> "$B_RTR_CURL_DATA; fi

	#A_RTR_CURL_DATA_USED=$(echo $A_RTR_CURL_DATA | jq -r '.gb')
	#B_RTR_CURL_DATA_USED=$(echo $B_RTR_CURL_DATA | jq -r '.gb')
	#.data_max_gb
	#A_RTR_CURL_DATA_MAX=$(echo $A_RTR_CURL_DATA | jq -r '.data_max_gb')
	#B_RTR_CURL_DATA_MAX=$(echo $B_RTR_CURL_DATA | jq -r '.data_max_gb')

	if [[ $PING_A_RES == 'OK'  ]] || [[ $A_RTR_CURL_DATA != '' ]]; then
		A_RTR_CURL_DATA_USED=$(echo $A_RTR_CURL_DATA | jq -r '.gb')
		A_RTR_CURL_DATA_MAX=$(echo $A_RTR_CURL_DATA | jq -r '.data_max_gb')
		A_DATA_USAGE_PERCENT=$(awk "BEGIN { pc=100*${A_RTR_CURL_DATA_USED}/${A_RTR_CURL_DATA_MAX}; i=int(pc); print (pc-i<0.5)?i:i+1 }")
	else
		A_DATA_USAGE_PERCENT=999
	fi

	if [[ $PING_B_RES == 'OK'  ]] || [[ $B_RTR_CURL_DATA != '' ]]; then
		B_RTR_CURL_DATA_USED=$(echo $B_RTR_CURL_DATA | jq -r '.gb')
		B_RTR_CURL_DATA_MAX=$(echo $B_RTR_CURL_DATA | jq -r '.data_max_gb')
		B_DATA_USAGE_PERCENT=$(awk "BEGIN { pc=100*${B_RTR_CURL_DATA_USED}/${B_RTR_CURL_DATA_MAX}; i=int(pc); print (pc-i<0.5)?i:i+1 }")
	else
		B_DATA_USAGE_PERCENT=999
	fi

	if [[ $1 == 'debug' ]]; then echo -e "A: "$A_RTR" - used reading: "$A_RTR_CURL_DATA_USED" / "$A_RTR_CURL_DATA_MAX" ("$A_DATA_USAGE_PERCENT"%)"; fi
	if [[ $1 == 'debug' ]]; then echo -e "B: "$B_RTR" - used reading: "$B_RTR_CURL_DATA_USED" / "$B_RTR_CURL_DATA_MAX" ("$B_DATA_USAGE_PERCENT"%)"; fi
}

# Add/Delete non/existing route
function ipRouteAddDel {

	IS_ROUTE=$(/sbin/route -n | grep $2 | grep UG | grep eth0 -c)
	LAST_MANIPULATION=$(cat /tmp/.routeManipulation.netBalancer.routing)
	if [[ $LAST_MANIPULATION == '' ]]; then
		# MANIPULATION_COOLDOWN
		date +%s > /tmp/.routeManipulation.netBalancer.routing
	else
		NOW_SECDATE=$(date +%s)
		LAST_MANIPULATION_DIFF=$((NOW_SECDATE-LAST_MANIPULATION))
		if [[ $LAST_MANIPULATION_DIFF -lt $MANIPULATION_COOLDOWN ]]; then
			logger "FAILURE: We're still waiting for cooldown ("$LAST_MANIPULATION_DIFF"/"$MANIPULATION_COOLDOWN")"
		else
			#logger "SUCCESS: We're cleared of cooldown ("$LAST_MANIPULATION_DIFF"/"$MANIPULATION_COOLDOWN")"
			if [[ $1 == 'add' ]]; then echo $2 > /root/routing/.netBalancer/using_node; fi

			if [[ $IS_ROUTE -eq 1 ]]; then
				# Route is present
				if [[ $1 == 'del' ]]; then
					# No need to add it, it is present
					# We can delete it tho 
					/sbin/ip route del default via $2
					if [[ $SILENCE -eq 0 ]]; then echo -e "\t --> deliting route: "$2; fi
					logger "\t -> deleting route: "$2
				else
					if [[ $SILENCE -eq 0 ]]; then echo -e "\t => No need to add it, already present."; fi
				fi
			else
				#route is not present
				if [[ $1 == 'add' ]]; then
					# No need to del it, it is not present
					# We can create it tho 
					#logger '/sbin/ip route add default via '$2
					/sbin/ip route add default via $2 > /root/stdout.txt 2> /root/stderr.txt
					if [[ $SILENCE -eq 0 ]]; then echo -e "\t ++> adding route: "$2; fi
					logger "\t +> adding route: "$2

					logger "Cooldown: Reseting cooldown counter."
					date +%s > /tmp/.routeManipulation.netBalancer.routing
				else
					if [[ $SILENCE -eq 0 ]]; then echo -e "\t => No need to del it, already present."; fi
				fi
			fi
		fi
	fi

}

# Choose most appropriate route to traver through
# Apply Route configuration
# Options / modes : 
#	- distributed		: will distribute between both 4G modems so consumption will go accordingnly
#	- fallback <a/b>	: will make sure that network will stay up and running at all time by netbalancing to slave node, 
#						  if network is OK on both nodes, the selected node will be the master and selected as default. By default this is Node A.

if [[ $OPT == "distributed" ]]; then
	#echo "##### Distributed : Route Manipulation #####" >> /root/logs/netBalancer.log
	logger "##### Distributed : Route Manipulation #####"
	findLessUsedProvider # Used to get default values for Data Used & Check Datamax & apply Soft Net Limitter.

	if [[ $A_UNSTABLE_SCORE -lt $B_UNSTABLE_SCORE ]]; then
		# A < B : donc A
		#echo "Choosing A --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE >> /root/logs/netBalancer.log
		logger "Choosing A --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE
		ipRouteAddDel del 192.168.0.252 #ip route del default via 192.168.0.252 #B
		ipRouteAddDel add 192.168.0.251 #ip route add default via 192.168.0.251 #A
	elif [[ $A_UNSTABLE_SCORE -gt $B_UNSTABLE_SCORE ]]; then
		# A > B : donc B
	    	#echo "Choosing B --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE >> /root/logs/netBalancer.log
	    	logger "Choosing B --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE
		ipRouteAddDel del 192.168.0.251 #ip route del default via 192.168.0.251 #A
		ipRouteAddDel add 192.168.0.252 #ip route add default via 192.168.0.252 #B
	elif [[ $A_UNSTABLE_SCORE -eq $B_UNSTABLE_SCORE ]]; then
		# A = B : Choose less used source
		#echo -e "Both sources are viable, finding the less used one..." >> /root/logs/netBalancer.log
		logger "Both sources are viable, finding the less used one..."
		findLessUsedProvider

		if [[ $A_DATA_USAGE_PERCENT -lt $B_DATA_USAGE_PERCENT ]]; then
			# A < B : A
			#echo -e "Node A is the less used --> A" >> /root/logs/netBalancer.log
			logger "Node A is the less used --> A \tA: "$A_RTR_CURL_DATA_USED"/"$A_RTR_CURL_DATA_MAX"Gb ("$A_DATA_USAGE_PERCENT"%) \tB: "$B_RTR_CURL_DATA_USED"/"$B_RTR_CURL_DATA_MAX"Gb ("$B_DATA_USAGE_PERCENT"%)"
		    ipRouteAddDel del 192.168.0.252 #ip route del default via 192.168.0.252 #B
		    ipRouteAddDel add 192.168.0.251 #ip route add default via 192.168.0.251 #A

		elif [[ $A_DATA_USAGE_PERCENT -gt $B_DATA_USAGE_PERCENT ]]; then
			# A > B : B
			#echo -e "Node B is the less used --> B" >> /root/logs/netBalancer.log
			logger "Node B is the less used --> B \tA: "$A_RTR_CURL_DATA_USED"/"$A_RTR_CURL_DATA_MAX"Gb ("$A_DATA_USAGE_PERCENT"%) \tB: "$B_RTR_CURL_DATA_USED"/"$B_RTR_CURL_DATA_MAX"Gb ("$B_DATA_USAGE_PERCENT"%)"
			ipRouteAddDel del 192.168.0.251 #ip route del default via 192.168.0.251 #A
			ipRouteAddDel add 192.168.0.252 #ip route add default via 192.168.0.252 #B

		else
			# A ?= B : A
			#echo -e "Both nodes looks the same or one is undefined... --> A" >> /root/logs/netBalancer.log
			logger "Both nodes looks the same or one is undefined... --> A"
			ipRouteAddDel del 192.168.0.252 #	ip route del default via 192.168.0.252 #B
			ipRouteAddDel add 192.168.0.251 #	ip route add default via 192.168.0.251 #A
		fi
	else
		#echo "Choosing A (by default) --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE >> /root/logs/netBalancer.log
		# or something not normal - Donc A
		logger "Choosing A (by default) --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE
		ipRouteAddDel del 192.168.0.252 #	ip route del default via 192.168.0.252 #B
		ipRouteAddDel add 192.168.0.251 #	ip route add default via 192.168.0.251 #A
	fi
elif [[ $OPT == "fallback" ]]; then
	#echo "##### Fallback : Route Manipulation ($ARG) #####" >> /root/logs/netBalancer.log
	logger "##### Fallback : Route Manipulation ($ARG) #####"
	findLessUsedProvider # Used to get default values for Data Used & Check Datamax & apply Soft Net Limitter.

	if [[ $A_UNSTABLE_SCORE -lt $B_UNSTABLE_SCORE ]]; then
		# A < B : donc A
		#echo "Choosing A --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE >> /root/logs/netBalancer.log
		logger "Choosing A --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE
	    ipRouteAddDel del 192.168.0.252 #ip route del default via 192.168.0.252 #B
	    ipRouteAddDel add 192.168.0.251 #ip route add default via 192.168.0.251 #A
	elif [[ $A_UNSTABLE_SCORE -gt $B_UNSTABLE_SCORE ]]; then
		# A > B : donc B
	    #echo "Choosing B --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE >> /root/logs/netBalancer.log
	    logger "Choosing B --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE
		ipRouteAddDel del 192.168.0.251 #ip route del default via 192.168.0.251 #A
		ipRouteAddDel add 192.168.0.252 #ip route add default via 192.168.0.252 #B
	else
		#echo "Choosing A (by default) --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE >> /root/logs/netBalancer.log
		# or something not normal - Donc A
		logger "Choosing A (by default) --> A="$A_UNSTABLE_SCORE", B="$B_UNSTABLE_SCORE
		if [[ $ARG == "b" ]]; then
			ipRouteAddDel del 192.168.0.251 #ip route del default via 192.168.0.251 #A
			ipRouteAddDel add 192.168.0.252 #ip route add default via 192.168.0.252 #B
		else
			ipRouteAddDel del 192.168.0.252 #	ip route del default via 192.168.0.252 #B
			ipRouteAddDel add 192.168.0.251 #	ip route add default via 192.168.0.251 #A
		fi
	fi
fi

#if [[ $SILENCE -eq 0 ]]; then
function display_result {
	echo "##### Result #####"
	USED_ROUTE=$(route -n | grep 192.168.0.25 | grep -v 192.168.0.250 | awk {'print $2'})
	#TRACE_ROUTE=$(traceroute facebook.com | head -n 2 | tail -n 1 | awk {'print $2$3$4$5'})
	TRACE_ROUTE=$(traceroute -m 1 facebook.com | tail -n 1 | awk {'print $2'})

	echo -e "\t--> Used Node: "$USED_ROUTE" ("$TRACE_ROUTE")"


	echo -e $WHI"+---+-------------------------------------------+--------+-----------+------+"$NC
	echo -e $WHI"| # | Node URL\t\t\t\t\t| Ping   | Usage     | %Use |"$NC
	echo -e $WHI"+---+-------------------------------------------+--------+-----------+------+"$NC
	echo -ne $WHI"| "

	if [[ $A_RTR_CURL == '' ]]; then
		echo -ne $DGRAY" # "$WHI"| "$DGRAY$B_RTR_CURL_URL$WHI"  | "$DGRAY" --- "$WHI"\t | "$DGRAY" --- "$WHI"\t | "$DGRAY" --- "$WHI"\t |"$NC"\n"
	else
		echo -ne "A "$WHI"| "$CYAN$A_RTR_CURL_URL$WHI"    | "
		if [[ $A_RTR_CURL_PING_STATUS == "OK" ]]; then
			echo -ne $LGR"OK"
		else
			echo -ne $RED"FAIL"
		fi
		echo -ne $WHI"\t | "
		if [[ $A_DATA_USAGE_PERCENT -lt 15 ]]; then
			COLOR=$CYAN
		elif [[ $A_DATA_USAGE_PERCENT -lt 25 ]]; then
			COLOR=$GREEN
		elif [[ $A_DATA_USAGE_PERCENT -lt 50 ]]; then
			COLOR=$LGR
		elif [[ $A_DATA_USAGE_PERCENT -lt 75 ]]; then
			COLOR=$YEL
		elif [[ $A_DATA_USAGE_PERCENT -lt 90 ]]; then
			COLOR=$LRED
		else
			COLOR=$RED
		fi
		echo -ne $COLOR$A_RTR_CURL_DATA_USED$WHI"\t"; if [[ $A_RTR_CURL_DATA_MAX -lt 100 ]]; then echo -ne '/ '$A_RTR_CURL_DATA_MAX; else echo -ne '/'$A_RTR_CURL_DATA_MAX; fi; echo -ne " |\t "$COLOR$A_DATA_USAGE_PERCENT$WHI" |"$NC"\n"
	fi

	if [[ $B_RTR_CURL == '' ]]; then
		echo -ne $DGRAY" # "$WHI"| "$DGRAY$B_RTR_CURL_URL$WHI"  | "$DGRAY" --- "$WHI"\t | "$DGRAY" --- "$WHI"\t | "$DGRAY" --- "$WHI"\t |"$NC"\n"
	else
		echo -ne $WHI"| B | "$CYAN$B_RTR_CURL_URL$WHI"    | ";

		if [[ $B_RTR_CURL_PING_STATUS == "OK" ]]; then
			echo -ne $LGR"OK"
		else
			echo -ne $RED"FAIL"
		fi
		echo -ne $WHI"\t | "
		if [[ $B_DATA_USAGE_PERCENT -lt 15 ]]; then
			COLOR=$CYAN
		elif [[ $B_DATA_USAGE_PERCENT -lt 25 ]]; then
			COLOR=$GREEN
		elif [[ $B_DATA_USAGE_PERCENT -lt 50 ]]; then
			COLOR=$LGR
		elif [[ $B_DATA_USAGE_PERCENT -lt 75 ]]; then
			COLOR=$YEL
		elif [[ $B_DATA_USAGE_PERCENT -lt 90 ]]; then
			COLOR=$LRED
		else
			COLOR=$RED
		fi
		echo -ne $COLOR$B_RTR_CURL_DATA_USED$WHI"\t"; if [[ $B_RTR_CURL_DATA_MAX -lt 100 ]]; then echo -ne '/ '$B_RTR_CURL_DATA_MAX; else echo -ne '/'$B_RTR_CURL_DATA_MAX;fi; echo -ne " |\t "$COLOR$B_DATA_USAGE_PERCENT$WHI" |"$NC"\n"
	fi

	echo -e $WHI"+---+-------------------------------------------+--------+-----------+------+"$NC
}

logger ""

if [[ $SILENCE -eq 0 ]]; then
	if [[ $OPT == "status" ]]; then
		findLessUsedProvider
	fi
	display_result
fi
